// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_wikidata.module;

const wikidataEntityUriPattern = 'http://www.wikidata.org/entity/Q([0-9]+)';
var wikidataEntityUriRegex = new RegExp(wikidataEntityUriPattern);

/// Find compound search suggestions using the Wikidata SPARQL endpoint.
///
/// The following SPARQL query is used:
///
///     PREFIX wd: <http://www.wikidata.org/entity/>
///     PREFIX wdt: <http://www.wikidata.org/prop/direct/>
///     PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
///
///     SELECT DISTINCT ?wdid (SAMPLE(?label) AS ?label) (SAMPLE(?inchikey) AS ?inchikey) WHERE {
///       { ?wdid wdt:P279 wd:Q11173 .} UNION { ?wdid wdt:P31 wd:Q11173 .}
///       ?wdid rdfs:label ?label .
///       ?wdid wdt:P235 ?inchikey .
///       FILTER(lang(?label) = "" || langmatches(lang(?label), "en")) .
///       FILTER (contains(lcase(?label), "penta"))
///     }
///     GROUP BY ?wdid
///     ORDER BY ASC(strlen(?label)) ASC(?wdid)
///     LIMIT 5
///
Future<v0.SearchSuggestions> findCompoundSuggestions(
    v0.FindSearchSuggestions query, SimpleState state) async {
  // Exit right away if the PubChem autocp provider is in accept or if the
  // query text is empty.
  if (!query.accept.contains(wikidataAutocpProvider) || query.text.isEmpty) {
    return new v0.SearchSuggestions(wikidataAutocpProvider, []);
  }

  // Build SPARQL query.
  var sparqlQuery = '''
PREFIX wd: <http://www.wikidata.org/entity/>
PREFIX wdt: <http://www.wikidata.org/prop/direct/>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>

SELECT DISTINCT ?wdid (SAMPLE(?label) AS ?label) (SAMPLE(?inchikey) AS ?inchikey) WHERE {
  { ?wdid wdt:P279 wd:Q11173 .} UNION { ?wdid wdt:P31 wd:Q11173 .}
  ?wdid rdfs:label ?label .
  ?wdid wdt:P235 ?inchikey .
  FILTER(lang(?label) = "" || langmatches(lang(?label), "${query.language}")) .
  FILTER (contains(lcase(?label), "${query.text.toLowerCase()}"))
}
GROUP BY ?wdid
ORDER BY ASC(strlen(?label)) ASC(?wdid)
LIMIT 5''';

  // Build request URL.
  var requestUrl =
      '$wikidataSparqlEndpoint?format=json&query=${Uri.encodeQueryComponent(sparqlQuery)}';

  // Execute request.
  Map<String, dynamic> response =
      JSON.decode((await state.httpClient.get(requestUrl)).body)
      as Map<String, dynamic>;

  // Extract suggestions.
  var suggestions = new List<SearchSuggestion>();
  if (response.containsKey('results') &&
      response['results'].containsKey('bindings')) {
    response['results']['bindings'].forEach((Map data) {
      // Get title.
      var title = data['label']['value'];

      // Get wdid.
      var match = wikidataEntityUriRegex.firstMatch(data['wdid']['value']);
      var wdid = int.parse(match.group(1));

      // Get inchikey.
      var inchikey = data['inchikey']['value'];

      suggestions.add(new SearchSuggestion(title, fields.entityCompound, [
        new FieldData(WikidataId.id, wdid),
        new FieldData(fields.InChIKey.id, inchikey)
      ]));
    });
  }

  return new v0.SearchSuggestions(wikidataAutocpProvider, suggestions);
}
